package renderer_file;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(create_renderer_file);


#-----------------------------------------------------------------------------
# modul renderer_file.pm
# part of the webconf-package for EIS/FAIR
# last modified: 2004/01/06
# (c) 2003-2004 by Maximilian Pasternak (maximilian.pasternak@bluescreen.de)
#-----------------------------------------------------------------------------

use strict;
use Fcntl ':flock';

my $separator = "-------------------------------------------------------------------------------";


##############################################################################
# sub create_renderer_file()
# - writes a new config-file
# IN:  $configfile - name of the config file
# OUT: renderer    - reference to hash with rendering functions
##############################################################################
sub create_renderer_file
{
   my $configfile = shift;
   my $configpath = "/etc/config.d/$configfile";

   ### build up hashtable for rendering configfiles
   my %renderer;

   ### open file
   open(CONF, ">$configpath") || die "Could not open '$configpath' for writing, reason: $!.\n";
   flock(CONF, LOCK_EX);

   ### add the interface
   $renderer{"call"}          = file_call(\%renderer);
   $renderer{"print"}         = file_print(\%renderer);

   ### default will be used if no specific function is provided
   $renderer{"print_default"} = file_print_default(\%renderer);

   ### empty line
   $renderer{"print_empty"}  = file_print_empty(\%renderer);

   ### header and footer
   $renderer{"print_header"} = file_print_header(\%renderer, $configfile);
#   $renderer{"print_footer"} = file_print_footer;  - not used for file

   ### groups
   $renderer{"begin_group"}  = file_begin_group(\%renderer);
   $renderer{"end_group"}    = file_end_group(\%renderer);

   ### lists
   $renderer{"begin_list"}   = file_print_empty(\%renderer);
   $renderer{"next_list"}    = file_print_empty(\%renderer);
#   $renderer{"end_list"}     = file_end_list;  - not used for file

   ### close renderer
   $renderer{"close"}        = file_close(\%renderer);

   ### read helpfile
   if (-f "/var/install/help/$configfile")
   {
      my $helptopic = "";
      my $helptext  = "";
      my %helphash;
      $renderer{"help"}       = \%helphash;

      my $Tag_help_on = "^[ \t]*<help name=\"(.*?)\">";
      my $Tag_help_off = '^[ \t]*<\/help>[ \t]*(#.*?)?\s*$';

      open(HELP, "</var/install/help/$configfile");
      LINE: while(<HELP>)
      {
         if (/$Tag_help_on/)
         {
            $helptopic = "$1";
            next LINE;
         }

         if ($helptopic)
         {
            if (/$Tag_help_off/)
            {
               $helphash{$helptopic} = $helptext;
               $helptopic = "";
               $helptext  = "";
               next LINE;
            }

            $helptext .= $_;
         }
      }
      close(HELP);
   }

   ### return renderer
   return \%renderer
}


##############################################################################
# sub renderer.call()
# - interface for calling the renderer
# IN:  $renderer  - this-pointer
#      $function  - function to be called
#      $param     - parameter for the function
# OUT: -
##############################################################################

sub file_call
{
my $renderer = shift;
return sub
{
   ### interface for this renderer
   my $function = shift || "";
   my $param    = shift || "";

   $renderer->{$function}->($param) if ($renderer->{$function});
}
}


##############################################################################
# sub renderer.print()
# - interface for printing elements
# IN:  $renderer  - this-pointer
#      $type      - type to render
#      $name      - name of the element
#      $value     - value of the element
#      $param     - hash with parameters
# OUT: -
##############################################################################

sub file_print
{
my $renderer = shift;
return sub
{
   ### interface for this renderer
   my $type     = shift || "default";
   my $name     = shift || "unknown";
   my $value    = shift;
   my $param    = shift;

   my $function = $renderer->{"print_$type"};
   $function    = $renderer->{"print_default"} unless ($function);

   $function->($name, $value, $param)           if ($function);
}
}


##############################################################################
#
# implementation of the renderer
#
#############################################################################

sub file_close
{
my $renderer = shift;
return sub
{
   flock(CONF, LOCK_UN);
   close(CONF);
}
}


sub file_print_header
{
my $renderer = shift;
my $service = shift;
return sub
{
   my $timestamp = localtime(time);

   # print the head of the config
   printf CONF "#$separator\n";
   printf CONF "# configurationfile for $service\n";
   printf CONF "# created by renderer_file.pm on $timestamp\n";
   printf CONF "#$separator\n\n";
}
}


sub file_begin_group
{
my $renderer = shift;
return sub
{
   my $name = shift;

   printf CONF "#$separator\n"    if ($name);
   printf CONF "# $name\n"        if ($name);
   printf CONF "#$separator\n\n";
}
}


sub file_end_group
{
my $renderer = shift;
return sub
{
   printf CONF "\n\n";
}
}


sub file_print_empty
{
my $renderer = shift;
return sub
{
   printf CONF "\n";
}
}


sub file_print_default
{
my $renderer = shift;
return sub
{
   my $name   = shift;
   my $value  = shift;

   my $oname  = @_[0]->{"name"} || "";

   my $hint   = @_[0]->{"hint"} || "";
   $hint = $renderer->{"help"}->{$oname} || "" unless ($hint);

   my $count;
   my $space;

   printf CONF "$name='$value'%n", $count;
   if ($hint)
   {
      printf CONF "\n%n ", $count              if ($count > 40);
      printf CONF "%*s # ", 40-$count, ' ';

      $space = 36;
      foreach $value (split(/\s+/, $hint))
      {
         if (length($value) > $space)
         {
            printf CONF "\n%*s # ", 40, '';
            $space = 36;
         }
         printf CONF "%s %n", $value, $count;
         $space = $space - $count;
      }  
   }
   printf CONF "\n";
}
}

1;

